//=============================================================================
//  MuseScore
//  Linux Music Score Editor
//  $Id:$
//
//  Tuning system plugin
//	Alter note tuning according to selected tuning system
//  To add a tuning system, you need to an element in the combo of tuningSystem.ui
//  Then, you need to define the difference in cents between the degrees of 
//  the "natural midi tuning" and the given tuning system in the tunings array below.
//
//  Copyright (C)2008 Werner Schweer and others
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================

//
// This is ECMAScript code (ECMA-262 aka "Java Script")
//

var form;

//---------------------------------------------------------
//    init
//    this function will be called on startup of mscore
//---------------------------------------------------------

function init()
     {
     // print("MIDI Cents init");
     }

//-------------------------------------------------------------------
//    run
//    this function will be called when activating the
//    plugin menu entry
//
//    global Variables:
//    pluginPath - contains the plugin path; file separator is "/"
//-------------------------------------------------------------------

function run()
{
	var loader = new QUiLoader(null);
	var file   = new QFile(pluginPath + "/TuningMIDICents.ui");
	file.open(QIODevice.OpenMode(QIODevice.ReadOnly, QIODevice.Text));
	form = loader.load(file, null);
	form.Apply.clicked.connect(accept);
	form.temperedPushButton.clicked.connect(toTempered);
	form.naturalPushButton.clicked.connect(toNatural);
	form.show();
}

function toTempered()
{
	form.noteCents.cDial.setValue(0);
	form.noteCents.dbDial.setValue(0);
	form.noteCents.dDial.setValue(0);
	form.noteCents.ebDial.setValue(0);
	form.noteCents.eDial.setValue(0);
	form.noteCents.fDial.setValue(0);
	form.noteCents.fsDial.setValue(0);
	form.noteCents.gDial.setValue(0);
	form.noteCents.abDial.setValue(0);
	form.noteCents.aDial.setValue(0);
	form.noteCents.bbDial.setValue(0);
	form.noteCents.bDial.setValue(0);
}

function toNatural()
{
	//This array contains the difference between natural tuning and tempered tuning systems
								//  C  Db   D   Eb   E    F   F#  G   Ab  A   Bb   B 
	const naturalTuning = [  0, -10,  4, -6, -16, -2, -12, 2, -8,  6,  -4, -14 ]; // natural
	var theTuning = new Array(12);
	var theOffset = (form.naturalComboBox.currentIndex + 1) % 12;
	
	for (var i=0; i<12; i++) { theTuning[i] = naturalTuning[ (i + 1 - theOffset + 12) % 12 ]; }
	form.noteCents.cDial.setValue(theTuning[0]);
	form.noteCents.dbDial.setValue(theTuning[1]);
	form.noteCents.dDial.setValue(theTuning[2]);
	form.noteCents.ebDial.setValue(theTuning[3]);
	form.noteCents.eDial.setValue(theTuning[4]);
	form.noteCents.fDial.setValue(theTuning[5]);
	form.noteCents.fsDial.setValue(theTuning[6]);
	form.noteCents.gDial.setValue(theTuning[7]);
	form.noteCents.abDial.setValue(theTuning[8]);
	form.noteCents.aDial.setValue(theTuning[9]);
	form.noteCents.bbDial.setValue(theTuning[10]);
	form.noteCents.bDial.setValue(theTuning[11]);
}

//---------------------------------------------------------
//    accept
//    called when user presses "Accept" button
//---------------------------------------------------------

function accept()
 {
	var theTuning = new Array(12);
	theTuning[0] = form.noteCents.cDial.value;
	theTuning[1] = form.noteCents.dbDial.value;
	theTuning[2] = form.noteCents.dDial.value;
	theTuning[3] = form.noteCents.ebDial.value;
	theTuning[4] = form.noteCents.eDial.value;
	theTuning[5] = form.noteCents.fDial.value;
	theTuning[6] = form.noteCents.fsDial.value;
	theTuning[7] = form.noteCents.gDial.value;
	theTuning[8] = form.noteCents.abDial.value;
	theTuning[9] = form.noteCents.aDial.value;
	theTuning[10] = form.noteCents.bbDial.value;
	theTuning[11] = form.noteCents.bDial.value;
	
	var cursor = new Cursor(curScore);
	curScore.startUndo();
	for (var staff = 0; staff < curScore.staves; ++staff)
	{
		cursor.staff = staff;
		for (var v = 0; v < 3; v++)
		{
		  cursor.voice = v;
		  cursor.rewind();  // set cursor to first chord/rest

		  while (!cursor.eos())
		  {
			  if (cursor.isChord())
			  {
				  for (var i = 0; i < cursor.chord().notes(); i++)
				  {
					  var note = cursor.chord().note(i);
					  var pitch = note.pitch % 12;
					  var name = note.name;
					  var idx = getIndex(pitch, name);
					  note.tuning = theTuning[idx];
				  }
			  }
			  cursor.next();
		  }
		}
	}
	curScore.endUndo();
 }

function getIndex(pitch, name){
    var result;
    switch (pitch) {
    	case 0: //C
    	   result = 0;
    	break;
    	case 1: //C# or Db
    	 result = 1;   
    	break;
    	case 2: //D
    	   result =2;
    	break;
    	case 3: //D# or Eb
    	   result = 3;   
    	break;
    	case 4: //E
    	   result = 4;
    	break;
    	case 5: //F
    	   result = 5;
    	break;
    	case 6: //F# or Gb
    	   result = 6;  
    	break;
    	case 7: //G
    	   result = 7;
    	break;
    	case 8: //G# or Ab
    	   result = 8;  
    	break;
    	case 9: //A
    	   result = 9;
    	break;
    	case 10: //A# or Bb
    	   result = 10;  
    	break;
    	case 11: //B
    	   result = 11;
    	break;
    	default: 
        result =0;
    	break;
  }
  return result;
}


//---------------------------------------------------------
//    menu:  defines were the function will be placed
//           in the MuseScore menu structure
//---------------------------------------------------------

var mscorePlugin = {
      menu: 'Plugins.MIDI Cent Tuning',
      init: init,
      run:  run
      };

mscorePlugin;

